  xdef INTR_SW_TSK
  xdef StartExecution
  xdef SaveSP
  
  
  ;Externally referenced symbols.
  xref currTask
  xref Tasks
  xref Stacks
  xref taskResolutionTable

; INTR_SW_TSK transfers execution to the next task ready to run.

INTR_SW_TSK:
    sei              ; Disable interrupts during task switch
    clrh
    ldx currTask
    aslx
    ldhx Stacks,x
    txs
    pulh
    cli
    rti


; This is the Helium scheduler.



StartExecution:
    ldx Tasks
    lda taskResolutionTable,x
    sta currTask
    asla
    tax
    ldhx Stacks,x    
    txs
    ais #1
    rti             ; Execute the task
    
; Saves the SP register in the Stacks[] array associated with currTask
SaveSP:
    tsx              ; preserve the value of the stack pointer in H:X before
    ais #-2          ; allocating space for a local variable on the stack
    sthx 1,sp        ; Store the value of the stack pointer in the stack space allocated
    lda 2,sp         ; Increment the stored value by 3 to account for the local variables allocated
    add #3           ; in TickISR (1 byte) and the return address for the function call (2 bytes)
    sta 2,sp
    bcc SPIncremented ; If there's no overflow to the next byte, just continue saving the SP
    lda 1,sp          ; If there is a carry, add it to the MSB of the stack pointer.
    adc #0
    sta 1,sp         ; Store the incremented value back on the stack
SPIncremented:
    clrh
    ldx currTask     ; Save the incremented value in the array
    aslx             ; element Stacks[currTask].
    lda 1,sp
    sta Stacks,x
    lda 2,sp
    sta Stacks+1,x
    ais #2           ; Deallocate stack frame and return.
    rts





