

  xdef TickISR
  xdef TickISRTaskSwitch
  
  xref TickBlock
  xref bitPosTable
  xref Tasks
  xref currTask
  xref taskResolutionTable
  xref INTR_SW_TSK
  XREF SaveSP
  

;      Stack frame used in TickISR
;
;  
;                 HIGH MEM
;
;      |---------------------------|
;      |            PCL            | <- SP BEFORE INTERRUPT
;      |---------------------------|
;      |            PCH            |
;      |---------------------------|
;      |             X             |
;      |---------------------------|
;      |             A             |
;      |---------------------------|
;      |            CCR            |
;      |---------------------------|
;      |             H             | <- SP AFTER INTERRUPT
;      |---------------------------|
;      |             i             | <- SP AFTER FIRST PSHH STATEMENT (first line of assembler)
;
;                 LOW MEM

TickISR:

    pshh
    pshh        ; Create a stack frame
    
    tsx
    lda #$ff       ; clear loop iterator
    sta ,sp
    
TimerDecrement:
    ldx ,sp        ; increment the iterator i
    incx
    cbeqx #$08, TickISRTaskSwitch  ; Since there are only 8 tasks, only count up to i=7. When i=8, we are done.
    stx ,sp

    clrh
    lda TickBlock,x     ; Load TickBlock[i] into register A
    beq TimerDecrement  ; If TickBlock[i] is already zero, go on to the next one.
    deca                ; Otherwise, decrement it
    sta TickBlock,x     ; And Store it
    bne TimerDecrement  ; And go on to the next array entry as long as the result of the decrement is nonzero.
    
    lda Tasks           ; If the result of the decrement is zero
    ldx ,sp             ; We need to set the corresponding task's
    clrh                ; ready bit in the variable Tasks to indicate
    ora bitPosTable,x   ; that the task is now ready to run.
    sta Tasks
    bra TimerDecrement
    
TickISRTaskSwitch:    
    ldx Tasks           ; Compare taskResolutionTable[Tasks] with currTask
    lda currTask        ; If the task that is currently running is the highest
    cmp taskResolutionTable,x
    beq TickISRDone     ; priority task ready to run, then just continue execution.
    
    jsr SaveSP          ; Save the current task's stack pointer
    ldx Tasks           ; Set currTask to point to the new task
    lda taskResolutionTable,x
    sta currTask
    jmp INTR_SW_TSK     ; Switch Tasks.

TickISRDone:
    pulh
    pulh
    rti    
    